package com.jakubu9333.grammarManager.config;

import com.jakubu9333.grammarManager.dao.XmlDataRepo;
import org.basex.BaseXServer;
import org.basex.api.client.ClientSession;
import org.basex.core.cmd.Add;
import org.basex.core.cmd.Check;
import org.basex.core.cmd.XQuery;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.env.Environment;
import org.springframework.http.HttpStatus;
import org.springframework.web.server.ResponseStatusException;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;


/**
 * @author Jakub Uhlarik
 */
@Configuration
public class XmlDataConfig {
    private XmlDataRepo xmlDataRepo;
    public static final String DB_NAME = "grammarsDB";
    @Autowired
    private Environment environment;

    @Bean
    public XmlDataRepo baseXMlDao() {
        ClientSession session;


        try {
            String host = environment.getProperty("DATABASE_HOST");
            String password = environment.getProperty("DATABASE_PASSWORD");
            if (password == null) {
                password = "admin";
            }
            if (host == null) {
                host = "localhost";
                new BaseXServer("-P" + password);
            }
            session = new ClientSession(host, 1984, "admin", password);
            System.out.println("\n\nConnecting to database at: " + host + "\n");
            session.execute(new Check(DB_NAME));
            System.out.println("\n\nFinding schema in databse\n\n");
            String result = session.execute(new XQuery("db:exists(\"grammarsDB\",\"grammar_check.xsd\")"));
            if (result.equals("false")) {
                String xmlSchema = Files.readString(Path.of("./xsd_schema/grammar_check.xsd"));
                System.out.println("\n\nSchema not found\n\n");

                session.execute(new Add("grammar_check.xsd", xmlSchema));
                System.out.println("\n\nLoading schema\n\n");
            }


        } catch (IOException e) {
            throw new ResponseStatusException(HttpStatus.INTERNAL_SERVER_ERROR, e.getMessage(), e);
        }

        this.xmlDataRepo = new XmlDataRepo(session);

        return this.xmlDataRepo;
    }

}
